function [ang_vel_x,ang_vel_y,ang_vel_z] = calculate_angular_velocity(ax1,ay1,az1,ax2,ay2,az2,dt)
    
    %Data sampling frequency
    fs = 1/dt;

    %-------------------------------------------------------------------------
    %Cube configuration
    %-------------------------------------------------------------------------

    L_x = 0.04; %6cm
    L_y = 0.04; %6cm
    L_z = 0.012; %1.2cm

    d_yz = sqrt(L_y^2 + L_z^2);
    d_xz = sqrt(L_x^2 + L_z^2);
    d_xy = sqrt(L_x^2 + L_y^2);

    angle_yz = asin(L_y/d_yz);
    angle_xz = asin(L_z/d_xz);
    angle_xy = asin(L_x/d_xy);
    
    angle_yz_comp = asin(L_z/d_yz);
    angle_xz_comp = asin(L_x/d_xz);
    angle_xy_comp = asin(L_y/d_xy);

    %-------------------------------------------------------------------------
    %Calculate angular rates
    %-------------------------------------------------------------------------
    
    accel_x1 = ax1;
    accel_x2 = ax2;
    
    accel_y1 = ay1;
    accel_y2 = ay2;
    
    accel_z1 = az1;
    accel_z2 = az2;
    
    tangential_accel_xy_x1 = accel_x1 * cos(angle_xy);
    tangential_accel_xy_x2 = accel_x2 * cos(angle_xy);
    
    tangential_accel_xy_y1 = accel_y1 * cos(angle_xy_comp);
    tangential_accel_xy_y2 = accel_y2 * cos(angle_xy_comp);

    tangential_accel_yz_y1 = accel_y1 * cos(angle_yz);
    tangential_accel_yz_y2 = accel_y2 * cos(angle_yz);
    
    tangential_accel_yz_z1 = accel_z1 * cos(angle_yz_comp);
    tangential_accel_yz_z2 = accel_z2 * cos(angle_yz_comp);

    tangential_accel_xz_z1 = accel_z1 * cos(angle_xz);
    tangential_accel_xz_z2 = accel_z2 * cos(angle_xz);
    
    tangential_accel_xz_x1 = accel_x1 * cos(angle_xz_comp);
    tangential_accel_xz_x2 = accel_x2 * cos(angle_xz_comp);
    
    %EcoIMU equations
    %{
    a_d_xy = (tangential_accel_xy_x1 - tangential_accel_xy_x2)/2;
    a_d_yz = (tangential_accel_yz_y1 - tangential_accel_yz_y2)/2;
    a_d_xz = (tangential_accel_xz_z1 - tangential_accel_xz_z2)/2;
    %}
    
    %Improved Equations
    a_d_xy = (tangential_accel_xy_y1 - tangential_accel_xy_y2 - (tangential_accel_xy_x1 - tangential_accel_xy_x2))/2;
    a_d_yz = (tangential_accel_yz_z1 - tangential_accel_yz_z2 - (tangential_accel_yz_y1 - tangential_accel_yz_y2))/2;
    a_d_xz = (tangential_accel_xz_x1 - tangential_accel_xz_x2 - (tangential_accel_xz_z1 - tangential_accel_xz_z2))/2;
    
    %Angular accleration normalized to unit radius
    alpha_dx = (2*a_d_yz)/d_yz;
    alpha_dy = (2*a_d_xz)/d_xz;
    alpha_dz = (2*a_d_xy)/d_xy;
    
    %Integrate to find angular velocity
    omega_dx = zeros(1, length(alpha_dx));
    omega_dy = zeros(1, length(alpha_dy));
    omega_dz = zeros(1, length(alpha_dz));

    for i = 2:length(alpha_dz)
       omega_dx(i) = omega_dx(i-1) + alpha_dx(i)*dt; 
       omega_dy(i) = omega_dy(i-1) + alpha_dy(i)*dt; 
       omega_dz(i) = omega_dz(i-1) + alpha_dz(i)*dt; 
    end
    
    ang_vel_x = omega_dx;
    ang_vel_y = omega_dy;
    ang_vel_z = omega_dz;
end

