clc;
clearvars;

%Use OpenGL to provide faster plot rendering
set(0, 'DefaultFigureRenderer', 'OpenGL');
set(gcf,'Renderer','OpenGL');
set(gcf,'RendererMode','manual');

%Close all open ports if any are open
if length(instrfindall)
    fclose(instrfindall);
    delete(instrfindall);
end

%Open COM port
s = serial('/dev/tty.usbmodem1411');
s.BaudRate = 230400;
fopen(s);

%Initiate Variables
time_interval = 0.00001; %time interval between each input.
x_end = 800;

time = 0;

x1 = 0;
y1 = 0;
z1 = 0;

x2 = 0;
y2 = 0;
z2 = 0;

gx = 0;
gy = 0;
gz = 0;

angle_x = 0;
angle_y = 0;
angle_z = 0;

%Set up the figures
figure_handle = figure('Name','Angular Velocity Z-Acceleration',...
    'Color',[1 1 1],'Visible','off');

axes_handle = axes('Parent',figure_handle,...
    'YGrid','on',...
    'YColor',[0 0 0],...
    'XGrid','on',...
    'XColor',[0 0 0],...
    'Color',[1 1 1]);

hold on;

plot_handle_x = plot(axes_handle, time, x1, 'Marker','.','Color',[1 0 0],'LineWidth', 1);
plot_handle_y = plot(axes_handle, time, y1, 'Marker','.','Color',[0 1 0],'LineWidth', 1);
plot_handle_z = plot(axes_handle, time, z1, 'Marker','.','Color',[0 0 1],'LineWidth', 1);

xlim(axes_handle, [0 x_end]);

%For measured acceleration
%ylim(axes_handle, [-20,20]);

%For angular velocity
ylim(axes_handle, [-250,250]);

xlabel('Samples');
ylabel('Angular Velocity (deg/s)');
title('Real Time Data');
legend('Accelerometers','Gyroscope');
set(figure_handle, 'Position', [1 1 800 600])

sample_count = 2;

while sample_count < x_end
    
    %Start plot from zero
    if isequal(sample_count,x_end)
       cla;
       
       clear time;
       clear x1;
       clear x2;
       
       sample_count = 2;
       time = 0;

       x1 = 0;
       y1 = 0;
       z1 = 0;

       x2 = 0;
       y2 = 0;
       z2 = 0;

       gx = 0;
       gy = 0;
       gz = 0;

       angle_x = 0;
       angle_y = 0;
       angle_z = 0;

       plot_handle_x = plot(axes_handle, time, x1, 'Marker','.','Color',[1 0 0]);
       plot_handle_y = plot(axes_handle, time, y1, 'Marker','.','Color',[0 1 0]);
       plot_handle_z = plot(axes_handle, time, z1, 'Marker','.','Color',[0 0 1]);
    end
    
    %Read incoming data from serial port
    str = fscanf(s,'%s');
    data = sscanf(str,'%f,');
    
    %Guard against invalid data
    if isequal(length(data),9)
        
        %Accelerometer 1 data (m/s^2)
        x1(sample_count) = data(1);
        y1(sample_count) = data(2);
        z1(sample_count) = data(3);
        
        %Accelerometer 2 data (m/s^2)
        x2(sample_count) = data(4);
        y2(sample_count) = data(5);
        z2(sample_count) = data(6);
        
        %Gyroscope data (deg/s)
        gx(sample_count) = data(7);
        gy(sample_count) = data(8);
        gz(sample_count) = data(9);
        
        %Calculate angular velocity
        [ang_vel_x,ang_vel_y,ang_vel_z] = calculate_angular_velocity(x1,y1,z1,x2,y2,z2,0.01);

        time(sample_count) = sample_count;
        
        %Integrate to find angle
        angle_x(sample_count) = angle_x(sample_count-1) + rad2deg(x1(sample_count))*0.032; 
        angle_y(sample_count) = angle_y(sample_count-1) + rad2deg(y1(sample_count))*0.032; 
        angle_z(sample_count) = angle_z(sample_count-1) + rad2deg(z1(sample_count))*0.032; 
        
        %Angle plot
        %set(plot_handle_x, 'YData', pos_z, 'XData', time);
        
        %Angular velocity plot
        set(plot_handle_x, 'YData', rad2deg(ang_vel_z), 'XData', time);
        set(plot_handle_y, 'YData', gz, 'XData', time);
        
        set(figure_handle, 'Visible', 'on');
        %drawnow update
        pause(time_interval);
        sample_count = sample_count + 1;
    end
end

%Store data
time = time';

x1 = x1';
y1 = y1';
z1 = z1';

x2 = x2';
y2 = y2';
z2 = z2';

gx = gx';
gy = gy';
gz = gz';

m = [time,x1,y1,z1,x2,y2,z2,gx,gy,gz];

csvwrite('simulation_data/angular_velocity_about_z_axis_100Hz_timer_isr_n.csv',m);

fclose(s);
delete(s);
clear s;